package com.dragoninfo.dcuc.authweb.restcontroller.api.controller;

import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ManufacturerInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.app.facade.IManufacturerInfoFacade;
import com.dragoninfo.dcuc.user.user.entity.PostInfo;
import com.dragoninfo.dcuc.user.user.entity.UserInfo;
import com.dragoninfo.dcuc.user.user.enumresources.UserTypeEnum;
import com.dragoninfo.dcuc.user.user.enumresources.YesNotEnum;
import com.dragoninfo.dcuc.user.user.facade.IAccountFacade;
import com.dragoninfo.dcuc.user.user.facade.IPostInfoFacade;
import com.dragoninfo.dcuc.user.user.facade.IUserFacade;
import com.dragoninfo.dcuc.user.user.facade.IUserInfoFacade;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.commons.util.ObjectUtils;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import java.lang.reflect.Field;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Administrator
 * @date 2018/6/25
 */
@Controller
@RequestMapping(value = "/api/")
public class ApiController {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    private static final String SERIAL_VERSION_UID = "serialVersionUID";
    private static final String ALIAS = "ALIAS";

    @Autowired
    private IUserFacade userFacade;

    @Autowired
    private IAccountFacade accountFacade;


    @Autowired
    private IManufacturerInfoFacade manufacturerInfoFacade;

    @Autowired
    private IApplyInfoFacade applyInfoFacade;

    @Autowired
    private IUserInfoFacade userInfoFacade;

    @Autowired
    private IPostInfoFacade postInfoFacade;




    /**
     * 查询用户
     * DcucLoginService 调用
     * @param id
     * @return
     */
    @RequestMapping(value = "users/id/{id}", method = RequestMethod.GET)
    @ResponseBody
    public Map userbyid(@PathVariable("id") String id) {
        UserInfo userInfo = userFacade.detailById(id);
        if (userInfo == null) {
            return null;
        }
        if (StringUtils.isBlank(userInfo.getId())) {
            return new HashMap();
        }
        String pSortNo = "0";
        if (StringUtils.equals(userInfo.getUserType(), "10")) {
            pSortNo = this.getPSortNo(userInfo.getId());
        }
        userInfo.setPSortNo(pSortNo);
        UserInfo user = userInfoFacade.userDetail(userInfo.getId());
        if (StringUtils.isNotEmpty(userInfo.getOrgId()) &&
                (UserTypeEnum.POLICE.getValue().equals(user.getUserType()) || userInfoFacade.getRootUser().equals(userInfo.getName()))) {
            SearchDTO searchDTO = new SearchDTO();
            Searchable searchable = Searchable.toSearchable(searchDTO);
            searchable.addSearchFilter("org_id", SearchOperator.eq, userInfo.getOrgId());
            searchDTO = searchable.toSearchDTO();
            searchDTO.setPage("1");
            searchDTO.setSize("2");
            Page<ApplyInfo> appPage = applyInfoFacade.applyInfoPage(searchDTO);
            userInfo.setCompany(CollectionUtils.isNotEmpty(appPage.getContent()) ? appPage.getContent().get(0).getManufacturerId() : "");
            ManufacturerInfo manufacturerInfo = manufacturerInfoFacade.manufacturerInfoDetail(userInfo.getCompany());
            userInfo.setCompanyCode(manufacturerInfo != null ? manufacturerInfo.getCode() : "");
        }
        Map map = resultMap(userInfo, user);
        return map;
    }


    /**
     * 校验登录
     * DcucLoginService.class 调用
     * @param account
     * @param password
     * @return
     */
    @RequestMapping(value = "/checklogin", method = RequestMethod.POST)
    @ResponseBody
    public ResponseDTO checklogin(String account, String password) {
        return accountFacade.checkLogin(account, password);
    }


    private String getPSortNo(String userId) {
        //获取t_post_info中的sortNo
        if (StringUtils.isEmpty(userId)){
            return "";
        }
        List<PostInfo> postInfos = postInfoFacade.postInfos(userId);
        if (postInfos != null && postInfos.size() > 0) {
            return postInfos.get(0).getSortNo().toString();
        } else {
            return "";
        }
    }


    private Map<String, Object> resultMap(UserInfo userInfo, UserInfo user) {
        Map<String, Object> map = this.beanToMapGD(userInfo);
        if (isParentOrg(userInfo, user)) {
            map.put("isParentOrg", YesNotEnum.YES.getValue());
        } else if (isParentOrg(userInfo, user) && !UserTypeEnum.POLICE.getValue().equals(userInfo.getUserType())) {
            map.put("isParentOrg", null);
        } else {
            map.put("isParentOrg", YesNotEnum.NO.getValue());
        }
        if (!ObjectUtils.isNullOrEmptyString(map.get("birth"))) {
            String birth = userInfo.getBirth();
            map.put("birth", birth);
        }
        if (!ObjectUtils.isNullOrEmptyString(map.get("createTime"))) {
            DateFormat bf = new SimpleDateFormat("yyyyMMdd");
            Date createTime = userInfo.getCreateTime();
            map.put("createTime", bf.format(createTime));
        }
        return map;
    }

    private boolean isParentOrg(UserInfo userInfo, UserInfo user) {
        return StringUtils.isNotEmpty(userInfo.getOrgId()) && userInfo.getOrgId().equals(user.getOrgId());
    }



    /**
     * api返回值为null时不转为""
     * @param obj
     * @param <T>
     * @return
     */
    public  <T> Map<String,Object> beanToMapGD(T obj){
        Map<String, Object> map = new HashMap<String, Object>();
        try {
            Class<?> clazz = obj.getClass();
            for (Field field : clazz.getDeclaredFields()) {
                field.setAccessible(true);
                String fieldName = field.getName();
                Object value = field.get(obj);
                if (!SERIAL_VERSION_UID.equals(fieldName) && !fieldName.startsWith(ALIAS) && !fieldName.endsWith(ALIAS)){
                    map.put(fieldName, (value != null) ? String.valueOf(value) : null);
                }
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return map;
    }
}
