package com.dragoninfo.dcuc.authweb.restcontroller.api.controller;

import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.auth.auth.dto.AppResourcesDto;
import com.dragoninfo.dcuc.auth.auth.dto.ApprovalDto;
import com.dragoninfo.dcuc.auth.auth.dto.RoleApiDto;
import com.dragoninfo.dcuc.auth.auth.facade.IApprovalFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IRoleFacade;
import com.dragoninfo.dcuc.auth.power.facade.IAppFunInfoFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.pojo.FunAppovalParamVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.pojo.RoleApiParamVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.pojo.UpdataAppFunVO;
import com.dragoninfo.dcuc.user.user.entity.UserInfo;
import com.dragoninfo.dcuc.user.user.facade.IUserInfoFacade;
import com.dragoninfo.duceap.core.enums.ResultEnum;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.HttpStatus;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Administrator
 * @date 2018/10/29
 */
@Controller
@RequestMapping(value = "/api")
public class AuthApiController {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IRoleFacade roleFacade;
    @Autowired
    private IApprovalFacade iApprovalFacade;
    @Autowired
    private IApplyInfoFacade applyInfoFacade;
    @Autowired
    private IAppFunInfoFacade iAppFunInfoFacade;
    @Autowired
    private IUserInfoFacade userInfoFacade;


    @RequestMapping(value = "/auth/getUserAuths", method = RequestMethod.GET)
    @ResponseBody
    public List<String> getAuthByUserIdAndAppCode(@RequestParam(value = "userId", required = true) String userId, @RequestParam(value = "appCode", required = true) String appCode) {
        return roleFacade.getAuthByUserIdAndAppCode(userId, appCode);
    }


    @RequestMapping(value = "/auth/user/{userId}/appCode/{appCode}", method = RequestMethod.GET)
    @ResponseBody
    public List<Map<String, String>> getAppAuthByUserIdAndAppCode(@PathVariable(value = "userId") String userId, @PathVariable(value = "appCode") String appCode) {
        return roleFacade.getAppAuthByUserIdAndAppCode(userId, appCode);
    }

    /**
     * 获取所有应用
     *
     * @return
     */
    @RequestMapping(value = "/auth/applications", method = RequestMethod.GET)
    @ResponseBody
    public List<ApplyInfo> applications() {
        return applyInfoFacade.getAllList();
    }

    @RequestMapping(value = "/roles/{appCode}/{userId}", method = RequestMethod.GET)
    @ResponseBody
    public ResponseDTO getRolesByAppCodeAndUserId(@PathVariable(value = "appCode") String appCode, @PathVariable(value = "userId") String userId) {
        return roleFacade.getRolesByAppCodeAndUserId(appCode, userId);
    }

    /**
     * 动态授权申请
     *
     * @param appCode        应用代码
     * @param roleCode       角色代码
     * @param userIdCard     申请人身份证
     * @param approvalIdCard 审批人身份证
     * @param reason         申请事由
     * @return
     */
    @RequestMapping(value = "/auth/apply", method = RequestMethod.GET)
    @ResponseBody
    public ResponseDTO proposer(
            @RequestParam(value = "appCode", required = true) String appCode,
            @RequestParam(value = "roleCode", required = true) String roleCode,
            @RequestParam(value = "userIdCard", required = true) String userIdCard,
            @RequestParam(value = "approvalIdCard", required = true) String approvalIdCard,
            @RequestParam(value = "reason", required = true) String reason) {
//        http://172.16.105.196:8888/dcuc/api/auth/apply?appCode=YHZX0000000000000001&roleCode=MANAGER_APP&userIdCard=000000000000000001&approvalIdCard=000000000000000001&reason=111
        return iApprovalFacade.authApply(appCode, roleCode, userIdCard, approvalIdCard, reason);
    }

    /**
     * 动态授权的角色功能获取
     *
     * @param appCode    应用代码
     * @param roleCode   角色代码
     * @param userIdCard 申请批人身份证
     * @return
     */
    @RequestMapping(value = "/auth/applyResult", method = RequestMethod.GET)
    @ResponseBody
    public ResponseDTO getProposerFuns(
            @RequestParam(value = "appCode", required = true) String appCode,
            @RequestParam(value = "roleCode", required = true) String roleCode,
            @RequestParam(value = "userIdCard", required = true) String userIdCard) {
        ApprovalDto approvalDto=new ApprovalDto();
        approvalDto.setAppCode(appCode);
        approvalDto.setRoleCode(roleCode);
        approvalDto.setUserIdCard(userIdCard);
//        http://172.16.105.196:8888/dcuc/api/auth/applyResult?appCode=YHZX0000000000000001&roleCode=MANAGER_APP&userIdCard=000000000000000001
        return iApprovalFacade.applyResult(approvalDto);
    }

    /**1.1
     * 授权管理服务 获取用户菜单权限列表
     * @param appCode
     * @return
     */
    @RequestMapping(value = "/getResources/{appCode}", method = RequestMethod.GET)
    @ResponseBody
    public Result getMenus(@PathVariable(value = "appCode") String appCode) {
        try {
            RoleApiDto dto=new RoleApiDto();
            dto.setAppCode(appCode);
            return Result.success( roleFacade.getMenus(dto));
        } catch (Exception e) {

            return Result.fail("获取失败");
        }
    }
    /**1.2
     * 推送应用功能资源
     * @return
     */
    @RequestMapping(value = "/setResources", method = RequestMethod.POST)
    @ResponseBody
    public Result updateAppFun(@RequestBody UpdataAppFunVO updataAppFunVO) {
        AppResourcesDto appResourcesDto=new AppResourcesDto(updataAppFunVO.getAppCode(),updataAppFunVO.getResources());
        try {
            ResponseDTO responseDTO = iAppFunInfoFacade.updateAppFun(appResourcesDto);
            if (ResponseDTO.FAIL_CODE.equals(responseDTO.getStatusCode())){
               return Result.fail(responseDTO.getMessage());
            }
            return  Result.success();
        } catch (Exception e) {
            logger.info("资源更新失败[{}]", e.getMessage());
            return Result.fail("资源更新失败", e.getMessage());
        }
    }

    /**1.3
     * 授权管理服务 获取用户角色权限列表
     * @return
     */
    @RequestMapping(value = "/getRoles", method = RequestMethod.POST)
    @ResponseBody
    public Result getRoles(@RequestBody RoleApiParamVo vo) {
        if(StringUtils.isEmpty(vo.getAppCode())){
            return Result.fail("appCode不能为空");
        }
        try {
            RoleApiDto dto=new RoleApiDto(vo.getAppCode(),vo.getIdcard(),vo.getRoleCode());
            return Result.success( roleFacade.getRoleByAppCodeAndUserId(dto));
        } catch (Exception e) {
            return Result.fail(ResultEnum.FAIL.getKey(),e.getMessage());
        }
    }


    /** 1.4
     * 动态授权
     * @param vo
     * @return
     */
    @RequestMapping("/pushData")
    @ResponseBody
    public Result pushData(@RequestBody FunAppovalParamVo vo) {
        String appCode = vo.getAppCode();
        String roleCode = vo.getRoleCode();
        String userIdCard = vo.getProposerIdcard();
        String approvalIdCard = vo.getApprpvalIdcard();
        String reason =vo.getProposerReason();
        if (StringUtils.isEmpty(appCode) || StringUtils.isEmpty(roleCode)
                || StringUtils.isEmpty(approvalIdCard) || StringUtils.isEmpty(userIdCard)
                ||StringUtils.isEmpty(vo.getVerifiy())||StringUtils.isEmpty(vo.getProposerOrg())) {
            return Result.fail("必填参数不能为空");
        }
        UserInfo proposerUserInfo = userInfoFacade.userDetail("idcard", userIdCard);
        if (proposerUserInfo==null||!proposerUserInfo.getOrgId().equals(vo.getProposerOrg())){
            return Result.fail("申请人不存在或机构错误");
        }
        ResponseDTO responseDTO = iApprovalFacade.authApply(appCode, roleCode, userIdCard, approvalIdCard, reason);
        if (ResponseDTO.FAIL_CODE.equals(responseDTO.getStatusCode())){
            return Result.fail(responseDTO.getMessage());
        }
        return Result.success();
    }



    /**1.5
     * 获取权限列表成功
     * @return
     */
    @RequestMapping(value = "/getRoleRight", method = RequestMethod.POST)
    @ResponseBody
    public Result getRoleRight(@RequestBody RoleApiParamVo vo) {

        if(StringUtils.isEmpty(vo.getAppCode())){
            return Result.fail("appCode不能为空");
        }
        try {
            RoleApiDto dto=new RoleApiDto(vo.getAppCode(),vo.getIdcard(),vo.getRoleCode());
            return Result.success(roleFacade.getRoleRight(dto));
        } catch (Exception e) {

            return Result.fail("获取失败");
        }
    }

    /**公共资源
     * 根据身份证ID找到这个赋予的应用
     * @param idcard
     * @return
     */

    @RequestMapping(value = "/apps/{idcard}", method = RequestMethod.GET)
    @ResponseBody
    public Result  appListByIdcard(@PathVariable("idcard") String idcard){
        try {
            return Result.success( roleFacade.appListByIdcard(idcard));
        } catch (Exception e) {
            return Result.fail("获取失败");
        }
    }


}
