package com.dragoninfo.dcuc.authweb.restcontroller.app;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.nacos.client.naming.utils.CollectionUtils;
import com.dragoninfo.dcuc.app.dto.ApplyInfoDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ManufacturerInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.app.facade.IManufacturerInfoFacade;
import com.dragoninfo.dcuc.auth.auth.vo.ApplicationInfoVO;
import com.dragoninfo.dcuc.auth.power.facade.IApplicationInfoFacade;
import com.dragoninfo.dcuc.authweb.common.MtAuthService;
import com.dragoninfo.dcuc.authweb.common.SysConstants;
import com.dragoninfo.dcuc.authweb.restcontroller.app.vo.AppVo;
import com.dragoninfo.dcuc.duceap.facade.IDuceapUploadFacade;
import com.dragoninfo.dcuc.org.facade.IOrgInfoFacade;
import com.dragoninfo.dcuc.org.vo.OrgTreeNode;
import com.dragoninfo.dcuc.user.user.entity.UserInfo;
import com.dragoninfo.dcuc.user.user.enumresources.UserTypeEnum;
import com.dragoninfo.dcuc.user.user.facade.IUserInfoFacade;
import com.dragoninfo.duceap.commons.util.server.OrgInfoUtil;
import com.dragoninfo.duceap.core.enums.ResultEnum;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.commons.util.tree.SimpleTreeNodeItemResolver;
import com.dragonsoft.duceap.commons.util.tree.TreeNodeUtils;
import com.dragonsoft.duceap.core.context.ContextUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * @author yica
 * @date 2018/8/8
 */
@Slf4j
@Api(tags = {"应用管理接口"})
@RestController
@RequestMapping(value = "/appsvr/v2")
public class AppController {

    @Autowired
    private IApplyInfoFacade applyInfoFacade;
    @Autowired
    private MtAuthService mtAuthService;
    @Autowired
    private IUserInfoFacade userInfoFacade;
    @Autowired
    private IOrgInfoFacade orgInfoFacade;
    @Autowired
    private IDuceapUploadFacade uploadFacade;
    @Autowired
    private IManufacturerInfoFacade manufacturerInfoFacade;
    @Autowired
    private IApplicationInfoFacade applicationInfoFacade;

    @ApiOperation(value = "查询应用列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件")
    @PostMapping(value = "apps")
    public Result<List<ApplyInfo>> applyPage(Searchable searchable) {
        UserInfo userInfo = userInfoFacade.userDetail(ContextUtils.getUserInfo().getId());
        String userType = userInfo.getUserType();
        String company = userInfo.getCompany();

        // 外部人员只能管理自己所属厂商下的应用
        if (UserTypeEnum.EXTERNAL.getValue().equals(userType)) {
            searchable.addSearchFilter("manufacturerId", SearchOperator.eq, company);
        }
        SearchDTO searchDTO = searchable.toSearchDTO();
        Page<ApplyInfo> page = applyInfoFacade.applyInfoPage(searchDTO);
        return Result.success(page.getTotalElements(), page.getContent());
    }

    @ApiOperation(value = "修改应用")
    @PutMapping(value = "/app/{id}")
    public Result<Object> update(@RequestBody AppVo appVo) {
        try {
            ApplyInfoDTO applyInfoDTO = new ApplyInfoDTO();
            BeanUtil.copyProperties(appVo, applyInfoDTO);
            //检查应用信息是否已存
            ResponseStatus responseStatus = applyInfoFacade.isRepeat(applyInfoDTO);
            if (!ResponseStatus.SUCCESS_CODE.equals(responseStatus.getStatusCode())) {
                return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), responseStatus.getMessage());
            }
            applyInfoFacade.updateApply(applyInfoDTO);
        } catch (Exception e) {
            log.error("update app", e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "修改异常");
        }
        return Result.success();
    }


    @ApiOperation(value = "保存应用")
    @PostMapping(value = "/app")
    public Result<Object> save(@RequestBody AppVo appVo) {
        try {
            ApplyInfoDTO applyInfoDTO = new ApplyInfoDTO();
            BeanUtil.copyProperties(appVo, applyInfoDTO);
            //检查应用信息是否已存
            ResponseStatus responseStatus = applyInfoFacade.isRepeat(applyInfoDTO);
            if (!ResponseStatus.SUCCESS_CODE.equals(responseStatus.getStatusCode())) {
                return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), responseStatus.getMessage());
            }
            applyInfoDTO.setOrgName(orgInfoFacade.detail(applyInfoDTO.getOrgId()).getFullName());

            applyInfoFacade.saveAppIy(applyInfoDTO);
        } catch (Exception e) {
            log.error("save app", e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "保存异常");
        }
        return Result.success();
    }

    @ApiOperation(value = "删除应用")
    @ApiImplicitParam(name = "id", value = "id")
    @DeleteMapping(value = "/app/{id}")
    public Result<Object> del(@PathVariable("id") String id) {
        try {
            boolean flag = userInfoFacade.extUsersByAppIdJudge(id);
            if (flag) {
                return Result.fail("300", "该应用下存在施工人员,不能删除");
            }
            applyInfoFacade.del(id);
        } catch (Exception e) {
            log.error("delete app error", e);
            throw new ApplicationException(300, "删除失败", e);
        }
        return Result.success("删除成功");
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "查询应用")
    @ApiImplicitParam(name = "id", value = "id")
    @GetMapping(value = "/app/{id}")
    public Result<AppVo> toDetail(@PathVariable("id") String id) {
        //查询应用信息
        ApplyInfo applyInfo = applyInfoFacade.applyDetail(id);
        //存入机构信息信息
        applyInfo.setOrgName(orgInfoFacade.detail(applyInfo.getOrgId()).getFullName());
        AppVo appVo = new AppVo();
        BeanUtil.copyProperties(applyInfo, appVo);
        appVo.setFile(uploadFacade.queryByBusiId(id, ""));
        return Result.success(appVo);
    }

    @ApiOperation(value = "开启停止应用")
    @ApiImplicitParam(name = "id", value = "id")
    @PutMapping(value = "/app/status/{id}")
    public Result<Object> startOrStop(@PathVariable String id) {
        String message = "";
        try {
            message = applyInfoFacade.startOrStop(id);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new ApplicationException(300, "操作失败", e);
        }
        return Result.success(message);
    }


    @ApiOperation(value = "根据厂商ID，获取厂商所有的应用")
    @ApiImplicitParam(value = "mfrId", name = "厂商Id", required = true)
    @GetMapping(value = "app/manufacturer/{mfrId}")
    public Result<List<Map<String, String>>> getAppByMfr(@PathVariable String mfrId) {
        if (StrUtil.isBlank(mfrId)) {
            return Result.success(Collections.emptyList());
        }

        List<ApplyInfo> appList = applyInfoFacade.getAppListByManufacturerId(mfrId);
        List<Map<String, String>> list = new ArrayList<>(appList.size());
        for (ApplyInfo app : appList) {
            Map<String, String> appmap = new HashMap<>();
            appmap.put("code", app.getId());
            appmap.put("value", app.getApplyName());
            appmap.put("managerName", app.getManagerName());
            appmap.put("managerPhone", app.getManagerPhone());
            list.add(appmap);
        }
        return Result.success(list);
    }

    @ApiOperation(value = "外部人员机构树")
    @PostMapping(value = "app/externalTree")
    public Result externalTree() {
        List<Map<String, Object>> result = new ArrayList<>();
        SecurityUser user = (SecurityUser) ContextUtils.getUserInfo();
        //用户范围
        String userMt = mtAuthService.getMtAuth(user.getId(), SysConstants.MT_USER);
        if (StringUtils.isEmpty(userMt)) {
            return Result.success(result);
        }
        //所有应用
        List<ApplyInfo> applyInfoList = applyInfoFacade.getAllList();
        //范围内加过的厂商
        Set<String> menuSet = new HashSet<>();
        List<String> applyIds = new ArrayList<>();
        for (ApplyInfo applyInfo : applyInfoList) {
            applyIds.add(applyInfo.getOrgId());
        }
        Map<String, OrgTreeNode> orgTreeNodes = orgInfoFacade.getOrgTreeNodeByList(applyIds);
        for (ApplyInfo applyInfo : applyInfoList) {
            OrgTreeNode node = orgTreeNodes.get(applyInfo.getOrgId());
            //应用所属机构在范围内
            if (node != null && OrgInfoUtil.isHaveAuth(node.getValue(), node.getPath(), userMt)) {
                Map<String, Object> appMap = new HashMap<>();
                appMap.put("id", applyInfo.getId());
                appMap.put("name", applyInfo.getApplyName());
                appMap.put("pid", applyInfo.getManufacturerId());
                appMap.put("isApply", true);
                result.add(appMap);
                //未添加的厂商
                if (!menuSet.contains(applyInfo.getManufacturerId())) {
                    ManufacturerInfo manu = manufacturerInfoFacade.manufacturerInfoDetail(applyInfo.getManufacturerId());
                    if (manu == null) {
                        continue;
                    }
                    menuSet.add(manu.getId());
                    Map<String, Object> map = new HashMap<>();
                    map.put("id", manu.getId());
                    map.put("name", manu.getManufacturerName());
                    map.put("isApply", false);
                    result.add(map);
                }
            }
        }
        result = TreeNodeUtils.generateTree(result, new SimpleTreeNodeItemResolver("id", "pid"));
        return Result.success(result);
    }

    /**
     * 根据机构Id获取应用信息列表
     **/
    @ApiOperation(value = "查询应用列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件")
    @PostMapping(value = "apps/role/_serach")
    public Result getAppList(Searchable searchable) {
        UserInfo userInfo = userInfoFacade.userDetail(ContextUtils.getUserInfo().getId());
        //外部人员只能配置自己所属厂商下的应用
        if (UserTypeEnum.EXTERNAL.getValue().equals(userInfo.getUserType())) {
            searchable.addSearchParam("manufacturerId", userInfo.getCompany());
        }
        SearchDTO searchDTO = searchable.toSearchDTO();
        Page<ApplyInfo> applyInfos = applyInfoFacade.applyInfoPage(searchDTO);
        if (CollectionUtils.isEmpty(applyInfos.getContent())) {
            return Result.success(0L, "");
        }
        //填充统计数据
        List<ApplicationInfoVO> apps = applicationInfoFacade.addCountInfo(objectConvert(applyInfos));
        return Result.success(applyInfos.getTotalElements(), apps);
    }

    private List<ApplicationInfoVO> objectConvert(Page<ApplyInfo> applyInfos) {
        List<ApplicationInfoVO> apps = new ArrayList<>();
        for (ApplyInfo applyInfo : applyInfos) {
            ApplicationInfoVO applicationInfoVO = new ApplicationInfoVO();
            applicationInfoVO.setId(applyInfo.getId());
            applicationInfoVO.setCode(applyInfo.getApplyCode());
            applicationInfoVO.setName(applyInfo.getApplyName());
            applicationInfoVO.setMenus(0L);
            applicationInfoVO.setRoles(0L);
            applicationInfoVO.setInitquotas(0L);
            applicationInfoVO.setQuotaed(0L);
            applicationInfoVO.setLeavequotas(0L);
            apps.add(applicationInfoVO);
        }
        return apps;
    }
}
