package com.dragoninfo.dcuc.authweb.restcontroller.auth;


import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.auth.auth.enumresources.YesNotEnum;
import com.dragoninfo.dcuc.auth.auth.vo.ApplicationInfoVO;
import com.dragoninfo.dcuc.auth.power.facade.IApplicationInfoFacade;
import com.dragoninfo.dcuc.authweb.config.DcucAuthWebConfig;
import com.dragoninfo.dcuc.authweb.restcontroller.auth.vo.ApplicationInfoVo;
import com.dragoninfo.dcuc.user.user.entity.UserInfo;
import com.dragoninfo.dcuc.user.user.enumresources.UserTypeEnum;
import com.dragoninfo.dcuc.user.user.facade.IUserInfoFacade;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.context.ContextUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@Api(tags = {"授权模块-应用管理接口"})
@RestController
@RequestMapping(value = "authsvr/v2/applicationinfo")
public class ApplicationController {

    public static Logger logger = LoggerFactory.getLogger(ApplicationController.class);

    @Autowired
    private DcucAuthWebConfig dcucAuthWebConfig;

    @Autowired
    private IApplicationInfoFacade iApplicationInfoFacade;
    @Autowired
    private IApplyInfoFacade applyInfoFacade;
    @Autowired
    private IUserInfoFacade userInfoFacade;


    /**
     * 根据机构Id获取应用信息列表
     *
     * @param
     * @return
     */
    @ApiOperation(value = "根据机构Id获取应用信息列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件,orgId不能为空")
    @PostMapping(value = "/appList/_search", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result<List<ApplicationInfoVo>> getAppList(SearchDTO searchDTO) {
        List<ApplicationInfoVo> resultList = new ArrayList<>();
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("deleted", SearchOperator.eq, YesNotEnum.NO.getValue());
        UserInfo userInfo = userInfoFacade.userDetail(ContextUtils.getUserInfo().getId());
        //外部人员只能配置自己所属厂商下的应用
        if (UserTypeEnum.EXTERNAL.getValue().equals(userInfo.getUserType())) {
            searchable.addSearchFilter("manufacturerId", SearchOperator.eq, userInfo.getCompany());
        }
        Page<ApplyInfo> applyInfos = applyInfoFacade.applyInfoPage(searchable.toSearchDTO());
        if (applyInfos.getContent().size() < 1) {
            return Result.success(new ArrayList<>());
        }
        //填充统计数据
        //TODO
        //DTO待设计
        List<ApplicationInfoVO> apps = iApplicationInfoFacade.addCountInfo(objectConvert(applyInfos));
        for (ApplicationInfoVO source : apps) {
            ApplicationInfoVo vo = new ApplicationInfoVo();
            BeanUtils.copyProperties(source, vo);
            resultList.add(vo);
        }
        return Result.success(applyInfos.getTotalElements(), resultList);
    }

    private List<ApplicationInfoVO> objectConvert(Page<ApplyInfo> applyInfos) {
        List<ApplicationInfoVO> apps = new ArrayList<ApplicationInfoVO>();
        for (ApplyInfo applyInfo : applyInfos) {
            ApplicationInfoVO applicationInfoVO = new ApplicationInfoVO();
            applicationInfoVO.setId(applyInfo.getId());
            applicationInfoVO.setCode(applyInfo.getApplyCode());
            applicationInfoVO.setName(applyInfo.getApplyName());
            applicationInfoVO.setMenus(0L);
            applicationInfoVO.setRoles(0L);
            applicationInfoVO.setInitquotas(0L);
            applicationInfoVO.setQuotaed(0L);
            applicationInfoVO.setLeavequotas(0L);
            applicationInfoVO.setAppOrgId(applyInfo.getOrgId());
            apps.add(applicationInfoVO);
        }
        return apps;
    }

    /**
     * 获取机构应用配额分配情况
     *
     * @param
     * @return
     */
    @ApiOperation(value = "根据机构Id获取应用信息列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件,orgId机构id不能为空")
    @PostMapping(value = "/appQutoList/_search")
    public Result<List<ApplicationInfoVo>> getAppQutoList(SearchDTO searchDTO) {
        List<ApplicationInfoVo> resultList = new ArrayList<>();
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("deleted", SearchOperator.eq, YesNotEnum.NO.getValue());
        Page<ApplyInfo> applyInfos = applyInfoFacade.applyInfoPage(searchable.toSearchDTO());
        if (applyInfos.getContent().size() < 1) {
            return Result.success(new ArrayList<>());
        }
        //填充统计数据
        //TODO
        //DTO待设计
        List<ApplicationInfoVO> apps = iApplicationInfoFacade.addCountInfo(objectConvert(applyInfos));
        for (ApplicationInfoVO source : apps) {
            ApplicationInfoVo vo = new ApplicationInfoVo();
            BeanUtils.copyProperties(source, vo);
            resultList.add(vo);
        }
        return Result.success(applyInfos.getTotalElements(), resultList);
    }

    /**
     * 获取所有机构应用
     *
     * @return
     */
    @ApiOperation(value = "获取所有机构应用")
    @ApiImplicitParam(name = "searchable", value = "查询条件")
    @PostMapping(value = "/appQutoList/all")
    public Result<List<ApplicationInfoVo>> getAllAppQutoList(SearchDTO searchDTO) {

        Searchable.toSearchable(searchDTO);
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("deleted", SearchOperator.eq, YesNotEnum.NO.getValue());
        searchable.addSearchFilter("applyStatus", SearchOperator.eq, YesNotEnum.NO.getValue());
        //TODO
        //DTO待设计
        Result result = getAppQutoListByBusiness(searchable);

        return result;
    }

    public Result getAppQutoListByBusiness(Searchable searchable) {
        int pageNum = searchable.getPageNumber() + 1;
        int size = searchable.getPageSize();
        Page<ApplyInfo> retainList = getApplyInfos(searchable);
//        int total = retainList.size();
//        //分页
//        if (total > size) {
//            if (pageNum == 1) {
//                retainList = retainList.subList(0, size);
//            } else {
//                int num = total - ((pageNum - 1) * size);
//                retainList = retainList.subList((pageNum - 1) * size, num > size ? (pageNum - 1) * size + size : (pageNum - 1) * size + num);
//            }
//
//        }
//        Page<ApplyInfo> page = new PageImpl<ApplyInfo>(retainList, new PageRequest(pageNum, size), total);
        //填充统计数据
        List<ApplicationInfoVO> apps = iApplicationInfoFacade.addCountInfo(objectConvert(retainList));
        List<ApplicationInfoVo> resultList = new ArrayList<>();
        for (ApplicationInfoVO source : apps) {
            ApplicationInfoVo vo = new ApplicationInfoVo();
            BeanUtils.copyProperties(source, vo);
            resultList.add(vo);
        }
        return Result.success( retainList.getTotalElements(), resultList);
    }

    /**
     * 根据登录人员警种，业务域获取应用
     * 应用保留  应用下拥有非治安警种角色的应用，应用下拥有角色业务域为空的应用，应用下的角色业务域和当前登录用户的业务域有交集的应用
     *
     * @param
     * @return
     */
    private Page<ApplyInfo> getApplyInfos(Searchable searchable) {
        String userId = ContextUtils.getUserInfo().getId();
        UserInfo userInfo = userInfoFacade.userDetail(userId);
        String userBusiness = userInfo.getPoliceBusiness();             //获取当前用户业务域
        if (userBusiness == null) {
            userBusiness = "";
        }
        searchable.addSearchFilter("userBusiness", SearchOperator.eq, userBusiness);
       // searchable.addSearchFilter("zaPolicectegory", SearchOperator.eq, dcucWebConfig.getZaPolicectegory());
        Page<ApplyInfo> retainList = applyInfoFacade.findApplyList(searchable.toSearchDTO());
        return retainList;
    }


}
