/**
 * 版权所有：厦门市巨龙信息科技有限公司
 * Copyright 2011 Xiamen Dragon Info. Tech. Co. Ltd.
 * All right reserved.
 * ====================================================
 * 文件名称: DownloadController.java
 * 修订记录：
 * No    日期				    作者(操作:具体内容)
 * 1.    2018-11-16			黄华源(创建:创建文件并实现模版下载)
 * ====================================================
 * 类描述：
 */
package com.dragoninfo.dcuc.authweb.restcontroller.download;

import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.system.ApplicationHome;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;

/**
 * <pre><b><font color="blue">DownloadController</font></b></pre>
 * <p/>
 * <pre><b>&nbsp;--描述说明--</b></pre>
 * <pre>
 *     提供警员模版、辅警模版、施工人员模版下载功能,采用io流实现。
 * </pre>
 * <pre>
 * <b>--样例--</b>
 *   前端发起请求:
 *      http://localhost:port/context/downloadTemplate/downPolice/{type}.action
 * </pre>
 * JDK版本：JDK1.6
 *
 * @author <b>黄华源</b>
 */

@Api(tags = {"附件下载接口"})
@RestController
@RequestMapping(value = "/downloadTemplate/")
public class DownloadController {

    public final String POLICE_TEMPLATE_RLPATH = "/template/PoliceTemplate.xls";
    public final String AUXILIARY_TEMPLATE_RLPATH = "/template/auxiliarypoliceTemplate.xls";
    public final String EXTERNALPERSONNEL_TEMPLATE_RLPATH = "/template/externalpersonnelTemplate.xls";
    public final String MANAGER_TEMPLATE_RLPATH = "/template/managerTemplate.xls";
    public final String GOVERNMENTPERSON_TEMPLATE_RLPATH = "/template/governmentpersonnelTemplate.xls";

    private Logger logger = LoggerFactory.getLogger(DownloadController.class);

    @ApiOperation(value = "获取xls附件")
    @ApiImplicitParam(name = "type", value = "附件类型：police 警员，auxiliaryPolice 辅警 " +
            "externalPersonnel 外部人员 manager 管理员 govUser 政府人员 ", required = true)
    @GetMapping(value = "/downTemplate")
    public HttpServletResponse downTemplate(@RequestParam("type") String type,  HttpServletResponse response) {
        ApplicationHome home = new ApplicationHome(getClass());
        File sysfile = home.getSource();
        String jarPath = sysfile.getPath();//classes路径
        if (StringUtils.equals(type, "police")) {
            response = down(response, POLICE_TEMPLATE_RLPATH);
        } else if (StringUtils.equals(type, "auxiliaryPolice")) {
            response = down(response, AUXILIARY_TEMPLATE_RLPATH);
        } else if (StringUtils.equals(type, "externalPersonnel")) {
            response = down(response, EXTERNALPERSONNEL_TEMPLATE_RLPATH);
        } else if (StringUtils.equals(type, "manager")) {
            response = down(response,MANAGER_TEMPLATE_RLPATH);
        } else if (StringUtils.equals(type, "govUser")) {
            response = down(response,  GOVERNMENTPERSON_TEMPLATE_RLPATH);
        }
        return response;
    }

    private HttpServletResponse down(HttpServletResponse response, String path) {
        try {
            // 以流的形式下载文件。
            Resource resource = new ClassPathResource(path);
            resource.getInputStream();
            InputStream fis = resource.getInputStream();
            byte[] buffer = new byte[fis.available()];
            fis.read(buffer);
            fis.close();
            // 清空response
            response.reset();
            // 设置response的Header
            String fileName = "";
            if ("PoliceTemplate.xls".equals( resource.getFilename())) {
                fileName = "警员模版.xls";
            } else if ("auxiliarypoliceTemplate.xls".equals( resource.getFilename())) {
                fileName = "辅警模版.xls";
            } else if ("externalpersonnelTemplate.xls".equals( resource.getFilename())) {
                fileName = "施工人员模版.xls";
            } else if ("managerTemplate.xls".equals( resource.getFilename())) {
                fileName = "管理员模板.xls";
            } else if ("governmentpersonnelTemplate.xls".equals( resource.getFilename())) {
                fileName = "政府人员模板.xls";
            }
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            response.addHeader("Content-Length", "" + resource.contentLength());
            OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
            response.setContentType("application/octet-stream");
            outputStream.write(buffer);
            outputStream.flush();
            outputStream.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return response;
    }

}
