package com.dragoninfo.dcuc.authweb.restcontroller.login;


import com.dragoninfo.dcuc.auth.auth.facade.IRoleFacade;
import com.dragoninfo.dcuc.authweb.config.DcucAuthWebConfig;
import com.dragoninfo.dcuc.org.facade.IOrgInfoFacade;
import com.dragoninfo.dcuc.user.admin.facade.IManageInfoFacade;
import com.dragoninfo.dcuc.user.admin.facade.IMenuMtAuthFacade;
import com.dragoninfo.dcuc.user.user.entity.UserInfo;
import com.dragoninfo.dcuc.user.user.enumresources.UserTypeEnum;
import com.dragoninfo.dcuc.user.user.facade.IUserInfoFacade;
import com.dragonsoft.duceap.base.entity.security.SecurityRight;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping(value = "/user")
public class DcucLoginController {

    @Autowired
    private IMenuMtAuthFacade iMenuMtAuthFacade;
    @Autowired
    private IUserInfoFacade iUserInfoFacade;
    @Autowired
    private IManageInfoFacade iManageInfoFacade;
    @Autowired
    private IRoleFacade iRoleFacade;
    @Autowired
    private IOrgInfoFacade iOrgInfoFacade;
    @Autowired
    private DcucAuthWebConfig dcucAuthWebConfig;

    public static final Logger logger = LoggerFactory.getLogger(DcucLoginController.class);

    public DcucLoginController() {
    }

    @GetMapping(value = "/info")
    public SecurityUser info() {
        UserInfo userInfo;
        try {
            SecurityUser securityUser = (SecurityUser) UserContextUtils.getCurrentUser();
            userInfo = iUserInfoFacade.userDetail(securityUser.getId());
            List<SecurityRight> authmenu = getAuthmenu(securityUser, userInfo);
            securityUser.setSecurityRightList(authmenu);
            //todo 因为数据库不同  后续不需要下面代码
            securityUser.setIdcard(userInfo.getIdcard());
            securityUser.setName(userInfo.getName());
            //ADMIN管理员
            String rootOrgid = userInfo.getOrgId();
            if (iUserInfoFacade.isRootUser(securityUser.getId()) && StringUtils.isEmpty(rootOrgid)) {
                rootOrgid = iOrgInfoFacade.getRootOrgId();
            }
            securityUser.setSecurityOrg(rootOrgid);
            return securityUser;
        } catch (Exception var3) {
            logger.error("获取用户信息异常", var3.getMessage());
            throw new SecurityException("获取用户信息异常", var3);
        }
    }


    //获取菜单
    private List<SecurityRight> getAuthmenu(SecurityUser securityUser, UserInfo userInfo) {
        List<SecurityRight> rights = new ArrayList<SecurityRight>();

        //用户中心菜单
        //ADMIN
        if (iUserInfoFacade.getRootUser().equals(securityUser.getName())) {
            rights = iMenuMtAuthFacade.getAllMenus("SysMenuInfo");
        } else if (dcucAuthWebConfig.isAuthAccess()) {
            logger.info("进入权限授权模块控制菜单权限");
            //使用权限授权模块控制菜单权限
            List<String> rightIds = iRoleFacade.getAuthByUserIdAndAppCode(securityUser.getId(), dcucAuthWebConfig.getAppCode());
            rights = iMenuMtAuthFacade.getMenuByIds(rightIds, "SysMenuInfo");
        } else {
            logger.info("进入管理员管理配置菜单权限");
            //使用管理员管理配置菜单权限
            rights = iMenuMtAuthFacade.menuForIndex(securityUser.getId(), "00000000000000000000000000000000");
            rights = menuFilter(userInfo, rights);
        }
        String[] codes = new String[rights.size()];
        for (int i = 0; i < rights.size(); i++) {
            String code = rights.get(i).getCode();
            codes[i] = code;
        }
        securityUser.setSecurityRights(codes);
        return rights;
    }


    private List<SecurityRight> menuFilter(UserInfo userInfo, List<SecurityRight> securityRightList) {

        List<SecurityRight> tempSearchList = new ArrayList<SecurityRight>();
        for (SecurityRight securityRight : securityRightList) {
            String code = securityRight.getCode();
            if ((!UserTypeEnum.POLICE.getValue().equals(userInfo.getUserType()) && !iUserInfoFacade.getRootUser().equals(userInfo.getName()))
                    && ("YHGLPT_ZHCX".equals(code) || "YHGLPT_ZHCX_RYCX".equals(code) || "YHGLPT_ZHCX_JGCX".equals(code))) {
                continue;
            }
            tempSearchList.add(securityRight);
        }
        securityRightList = tempSearchList;

        return securityRightList;
    }

}
