package com.dragoninfo.dcuc.authweb.restcontroller.org;

import com.dragoninfo.dcuc.authweb.restcontroller.org.vo.GovOrgVo;
import com.dragoninfo.dcuc.authweb.restcontroller.org.vo.SortVo;
import com.dragoninfo.dcuc.org.entity.GovInfo;
import com.dragoninfo.dcuc.org.facade.IGovInfoFacade;
import com.dragoninfo.dcuc.user.entity.GovUserInfo;
import com.dragoninfo.dcuc.user.facade.IGovUserInfoFacade;
import com.dragoninfo.dcuc.user.user.enumresources.YesNotEnum;
import com.dragoninfo.dcuc.user.user.facade.IUserInfoFacade;
import com.dragoninfo.duceap.core.enums.ResultEnum;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.commons.util.bean.BeanUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.context.ContextUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 勿删权限政务机构列表
 * Created by Administrator on 2020/6/30.
 */
@Api(tags = {"政务机构管理接口"})
@RestController
@RequestMapping(value = "/orgsvr/")
public class GovernmentInfoController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IGovInfoFacade iGovInfoFacade;
    @Autowired
    private IUserInfoFacade iUserInfoFacade;
    @Autowired
    private IGovUserInfoFacade iGovUserInfoFacade;

    @ApiOperation(value="政务机构新增")
    @PostMapping("govOrg")
    public Result save(@RequestBody GovOrgVo vo){
        try{
            GovInfo government = new GovInfo();
            BeanUtils.copyProperties(government,vo);
            if(StringUtils.isBlank(government.getUpGovId())){
                government.setIsRoot("0");
            }else{
                government.setIsRoot("1");
            }
            SecurityUser user = (SecurityUser) ContextUtils.getUserInfo();
            if (iGovInfoFacade.checkCode("", government.getCode()))
                return Result.fail(ResultEnum.FAIL.getKey(), "政务机构代码已存在");
            if (iGovInfoFacade.checkName("", government.getFullName(), ""))
                return Result.fail(ResultEnum.FAIL.getKey(), "政务机构名称已存在");
            if (!iUserInfoFacade.getRootUser().equals(user.getUserName()) && StringUtils.isEmpty(government.getUpGovId()))
                return Result.fail(ResultEnum.FAIL.getKey(), "请选择机构后再新增！");
            iGovInfoFacade.addGov(government);
        }catch (Exception e){
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"新增异常");
        }
        return Result.success();
    }

    @ApiOperation(value="政务机构修改")
    @PutMapping("govOrg/{id}")
    public Result update(@RequestBody GovOrgVo vo){
        try{
            GovInfo government = new GovInfo();
            BeanUtils.copyProperties(government,vo);
            government = setUpdateProperties(government);
            iGovInfoFacade.updateGov(government);
        }catch(Exception e){
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"修改异常");
        }
        return Result.success();
    }

    @ApiOperation(value = "删除政务机构")
    @DeleteMapping("govOrg/{id}")
    public Result delete(@PathVariable("id") String id){
        Searchable searchable = Searchable.newSearchable();
        searchable.addSearchFilter("gov_id", SearchOperator.eq,id);
        searchable.addSearchFilter("deleted",SearchOperator.eq, YesNotEnum.NO.getValue());
        SearchDTO searchDTO = searchable.toSearchDTO();
        Page<GovUserInfo> page = iGovUserInfoFacade.govUserInfoPage(searchDTO);
        if (page.getContent().size() > 0)
            return Result.fail("该机构有人员存在，不可删除");
        iGovInfoFacade.deleteOrg(id);
        return Result.success();
    }

    @ApiOperation(value="获取政务机构")
    @GetMapping(value = "govOrg/{id}")
    public Result get(@PathVariable("id") String id){
        GovInfo government = new GovInfo();
        government = iGovInfoFacade.getGovInfoById(id);
        GovOrgVo vo = new GovOrgVo();
        try {
            BeanUtils.copyProperties(vo,government);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            return Result.fail("属性复制异常");
        }
        return Result.success(vo);
    }


    @ApiOperation(value="政务机构树")
    @ApiImplicitParam(name = "id",value = "id")
    @PostMapping(value = "govOrg/tree/full")
    public Result fullGovTree(@RequestBody Map params) {
        String id = (String) params.get("id");
        if (StringUtils.isEmpty(id)){
            //根节点
            GovInfo root = iGovInfoFacade.getRootGov();
            List<Map<String, Object>> resultList = new ArrayList<Map<String, Object>>();
            resultList.add(treeMapFilter(root,null,""));
            return Result.success(resultList);
        }
        return  Result.success(getChildTreeNode(id, YesNotEnum.NO.getValue()));
    }

    @ApiOperation(value = "政务机构拖动排序")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "startId", value = "需要拖动的机构", required = true),
            @ApiImplicitParam(name = "endId", value = "拖动到目标机构", required = true),
            @ApiImplicitParam(name = "moveType", value = "prev:目标机构前,next: 目标机构后", required = true)
    })
    @PostMapping(value = "govOrg/sort")
    public Result reSort(@RequestBody SortVo sortVo){
        try{
            iGovInfoFacade.reSort(sortVo.getStartId(), sortVo.getEndId(), sortVo.getMoveType());
        }catch (Exception e){
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"排序异常");
        }

        return Result.success();
    }

    @ApiOperation(value = "政务机构树匹配查询")
    @ApiImplicitParams({@ApiImplicitParam(name = "name", value = "机构名称", required = true),
            @ApiImplicitParam(name = "pageNum", value = "条数", required = true),
            @ApiImplicitParam(name = "pageSize", value = "页数", required = true)
    })
    @PostMapping(value = "govOrg/tree/match")
    public Result govMatch( @RequestBody Map map){
        String name =map.get("name").toString();
        Integer pageNum =Integer.valueOf(map.get("pageNum").toString());
        Integer pageSize =Integer.valueOf(map.get("pageSize").toString());
        return Result.success(iGovInfoFacade.govMatch(name,pageNum,pageSize).getResult());
    }

    private GovInfo setUpdateProperties(GovInfo government){
        GovInfo oldGov = new GovInfo();
        if(StringUtils.isNotBlank(government.getId())){
            oldGov = iGovInfoFacade.getGovInfoById(government.getId());
        }
        oldGov.setId(government.getId());
        oldGov.setCode(government.getCode());
        oldGov.setShortName(government.getShortName());
        oldGov.setFullName(government.getFullName());
        oldGov.setUpGovName(government.getUpGovName());
        oldGov.setUpGovId(government.getUpGovId());
        return oldGov;
    }

    /**
     * 组装树节点
     *
     * @param government
     * @param orgNums
     * @param state   节点勾选状态
     * @return
     */
    private Map<String, Object> treeMapFilter(GovInfo government, Map<String, Long> orgNums, String state) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("id", government.getId());
        map.put("code", government.getCode());
        map.put("name", StringEscapeUtils.unescapeHtml4(government.getFullName()));
        map.put("pid", government.getUpGovId());
        map.put("isParent", true);
        if (null == orgNums){
            map.put("isParent", true);
            return map;
        }
        if ("0".equals(state) || null == orgNums.get(government.getId()) || 0 == orgNums.get(government.getId())) {
            map.put("isParent", false);
        } else {
            map.put("isParent", true);
        }
        return map;
    }


    /**
     * 异步获取子节点
     *
     * @param parentId
     * @param state    机构状态：0：未删除，1：已删除，null：全部
     * @return
     */
    public List<Map<String, Object>> getChildTreeNode(String parentId, String state) {
        List<Map<String, Object>> resultList = new ArrayList<Map<String, Object>>();
        //子级机构
        List<GovInfo> childList = iGovInfoFacade.findChildGovList(parentId);
        //子级和自身的子级个数
        Map<String, Long> orgNums = iGovInfoFacade.countGovByUpGovId(parentId);
        for (GovInfo government : childList) {
            Map<String, Object> map = treeMapFilter(government, orgNums, null);
            resultList.add(map);
        }
        return resultList;
    }
}
