package com.dragoninfo.dcuc.authweb.restcontroller.user.manager.utils;


import com.dragonsoft.duceap.base.entity.http.ResponseStatus;

/**
 * @description: 密码校验
 * @author: hognzp
 * @time: 2019/8/27
 **/
public class CheckPassword {

    //数字
    public static final String REG_NUMBER = ".*\\d+.*";
    //小写字母
    public static final String REG_UPPERCASE = ".*[A-Z]+.*";
    //大写字母
    public static final String REG_LOWERCASE = ".*[a-z]+.*";
    //特殊符号(~!@#$%^&*()_+|<>,.?/:;'[]{}\)
    public static final String REG_SYMBOL = ".*[~!@#$%^&*()_+|<>,.?/:;'\\[\\]{}\"]+.*";


    /**
     * @description: 密码强度校验
     * 1) 密码控制只能输入字母、数字、特殊符号(~!@#$%^&*()_+[]{}|\;:'",./<>?)
     * 2) 长度 8-30 位，必须包括大小写字母、数字、特殊符号中的3种
     * @author: hognzp
     * @time: 2019/8/27
     **/
    public static ResponseStatus checkPasswordRule(String password){

        //密码为空及长度大于8位小于30位判断
        if (password == null || password.length() <8 || password.length()>25)
            return ResponseStatus.fail("密码长度应大于等于8位小于25位");
        int i = 0;

        if (password.matches(CheckPassword.REG_NUMBER)) i++;
        if (password.matches(CheckPassword.REG_LOWERCASE))i++;
        if (password.matches(CheckPassword.REG_SYMBOL)) i++;
        if(password.matches(CheckPassword.REG_UPPERCASE)) i++;
        if (i  < 3 )
            return ResponseStatus.fail("新密码必须包含大/小写字母、数字、特殊符号!");

        return ResponseStatus.success();
    }

    public static void main(String[] args) {
        boolean flag = ResponseStatus.SUCCESS_CODE.equals(checkPasswordRule("$f33S3333333").getStatusCode());
        System.out.println(flag);
    }
}
