package com.dragoninfo.dcuc.authweb.restcontroller.user.manager.utils;


import com.dragoninfo.dcuc.auth.auth.enumresources.YesNotEnum;
import com.dragoninfo.dcuc.authweb.util.UserUtils;
import com.dragoninfo.dcuc.user.entity.GovUserInfo;
import com.dragoninfo.dcuc.user.user.entity.UserInfo;
import com.dragoninfo.dcuc.user.user.enumresources.UserStatusEnum;
import com.dragoninfo.dcuc.user.user.enumresources.UserTypeEnum;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by yica on 2018/6/15.
 */
public class ImpExcelUtils {


    /**
     * 导入数据
     *
     * @param inputStream
     * @return
     * @throws IOException
     */
    public static List<Map<String, String>> loadAdminInfo(InputStream inputStream) throws IOException, NumberFormatException {
        List<Map<String, String>> adminMapList = new ArrayList<Map<String, String>>();
        //根据指定的文件输入流导入Excel从而产生Workbook对象
        Workbook wb0 = new HSSFWorkbook(inputStream);
        //获取Excel文档中的第一个表单
        Sheet sht0 = wb0.getSheetAt(0);
        //对Sheet中的每一行进行迭代
        for (Row r : sht0) {
            Cell cell0 = r.getCell(0);
            Cell cell1 = r.getCell(1);
            Cell cell2 = r.getCell(2);
            Map<String, String> adminMap = new HashMap<String, String>();

            if (r.getRowNum() == 0) {
                if (!cell0.getStringCellValue().equals("身份证号")) {
                    throw new NumberFormatException("缺少''身份证号''列！");
                }
                if (!cell1.getStringCellValue().equals("机构代码（不填默认为目标警员所在单位）")) {
                    throw new NumberFormatException("缺少''机构代码（不填默认为目标警员所在单位）''列！");
                }
                if (!cell2.getStringCellValue().equals("权限范围（0：本单位，2：本单位及下级单位）")) {
                    throw new NumberFormatException("缺少''权限范围''列！");
                }
                continue;
            }

            //取出当前行第1个单元格数据
            if (cell0 == null || StringUtils.isBlank(cell0.getStringCellValue())) {
                if ((cell1 == null || StringUtils.isBlank(cell1.getStringCellValue())) && (cell2 == null || StringUtils.isBlank(cell2.getStringCellValue()))) {
                    continue;
                }
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''身份证''为空！");
            }
            if (!UserUtils.isIDNumber(cell0.getStringCellValue())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''身份证''格式错误！");
            }
            adminMap.put("idCard", cell0.getStringCellValue());

            //取出当前行第2个单元格数据
            if (cell1 == null || StringUtils.isBlank(cell1.getStringCellValue())) {
                adminMap.put("code", "");
            } else {
                adminMap.put("code", cell1.getStringCellValue());
            }

            //取出当前行第3个单元格数据
            if (cell2 == null || StringUtils.isBlank(cell2.getStringCellValue())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''权限范围''为空");
            }
            if (!"0".equals(cell2.getStringCellValue()) && !"2".equals(cell2.getStringCellValue())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''权限范围''有误！应为0或2！");
            }
            adminMap.put("auth", cell2.getStringCellValue());

            adminMapList.add(adminMap);
        }
        return adminMapList;
    }

    /**
     * 导入数据
     *
     * @param inputStream
     * @param userType
     * @return
     * @throws IOException
     */
    public static List<UserInfo> loadScoreInfo(InputStream inputStream, String userType) throws IOException, NumberFormatException {
        List<UserInfo> temp = new ArrayList();
        //根据指定的文件输入流导入Excel从而产生Workbook对象
        Workbook wb0 = new HSSFWorkbook(inputStream);
        //获取Excel文档中的第一个表单
        Sheet sht0 = wb0.getSheetAt(0);
        //对Sheet中的每一行进行迭代
        for (Row r : sht0) {
            //如果当前行的行号（从0开始）未达到2（第三行）则从新循环
            if (r.getRowNum() == 0) {
                if (!r.getCell(0).getStringCellValue().equals("姓名")) {
                    throw new NumberFormatException("缺少''姓名''列！");
                }
                if (!r.getCell(2).getStringCellValue().equals("身份证")) {
                    throw new NumberFormatException("缺少''身份证''列！");
                }
                int cellNum = 3;
                if (UserTypeEnum.POLICE.getValue().equals(userType) || UserTypeEnum.HELPER.getValue().equals(userType)) {
                    if (!r.getCell(cellNum++).getStringCellValue().equals("单位机构代码")) {
                        throw new NumberFormatException("缺少''单位机构代码''列！");
                    }
                }
                if (!r.getCell(cellNum++).getStringCellValue().equals("手机号码（接收短信验证码）")) {
                    throw new NumberFormatException("缺少''手机号码（接收短信验证码）''列！");
                }
                if (!r.getCell(cellNum++).getStringCellValue().equals("民族（表码）")) {
                    throw new NumberFormatException("缺少''民族（表码）''列！");
                }
                if(UserTypeEnum.EXTERNAL.getValue().equals(userType)){
                    if (!r.getCell(cellNum++).getStringCellValue().equals("QQ号")) {
                        throw new NumberFormatException("缺少''QQ号''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("e-mail")) {
                        throw new NumberFormatException("缺少''e-mail''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("微信号")) {
                        throw new NumberFormatException("缺少''微信号''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("座机")) {
                        throw new NumberFormatException("缺少''座机''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("联系地址")) {
                        throw new NumberFormatException("缺少''联系地址''列！");
                    }
                }
                if (UserTypeEnum.EXTERNAL.getValue().equals(userType)) {
                    //施工人员
                    if (!r.getCell(1).getStringCellValue().equals("编号")) {
                        throw new NumberFormatException("缺少''编号''列！");
                    }
                } else if (UserTypeEnum.POLICE.getValue().equals(userType)) {
                    //警员
                    if (!r.getCell(1).getStringCellValue().equals("警号")) {
                        throw new NumberFormatException("缺少''警号''列！");
                    }
                    cellNum = 6;
                    if (!r.getCell(cellNum++).getStringCellValue().equals("手机号码（私人）")) {
                        throw new NumberFormatException("缺少''手机号码（私人）''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("人员类型（表码）")) {
                        throw new NumberFormatException("缺少''人员类型（表码）''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("警衔（表码）")) {
                        throw new NumberFormatException("缺少''警衔（表码）''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("职级（表码）")) {
                        throw new NumberFormatException("缺少''职级（表码）''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("职务")) {
                        throw new NumberFormatException("缺少''职务''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("在职类型（表码）")) {
                        throw new NumberFormatException("缺少''在职类型（表码）''列！");
                    }
                    if (!r.getCell(cellNum++).getStringCellValue().equals("警种（表码）")) {
                        throw new NumberFormatException("缺少''警种（表码）''列！");
                    }
/*                    if (!r.getCell(18).getStringCellValue().equals("业务域（表码）")) {
                        throw new NumberFormatException("缺少\'\'业务域（表码）\'\'列！");
                    }*/
                } else if (UserTypeEnum.HELPER.getValue().equals(userType)) {
                    //辅警
                    if (!r.getCell(1).getStringCellValue().equals("辅警编号")) {
                        throw new NumberFormatException("缺少''辅警编号''列！");
                    }
                    if (!r.getCell(6).getStringCellValue().equals("人员类型（表码）")) {
                        throw new NumberFormatException("缺少''人员类型（表码）''列！");
                    }
                    if (!r.getCell(7).getStringCellValue().equals("警种(表码)")) {
                        throw new NumberFormatException("缺少''警种(表码)''列！");
                    }
                    if (!r.getCell(8).getStringCellValue().equals("人员状态(表码)")) {
                        throw new NumberFormatException("缺少''人员状态(表码)''列！");
                    }
                }
                continue;
            }
            //创建实体类
            UserInfo info = new UserInfo();
            //取出当前行第1个单元格数据，并封装在info实体stuName属性上
            if (r.getCell(0) == null || StringUtils.isBlank(r.getCell(0).getStringCellValue())) {
                if ((r.getCell(1) == null || StringUtils.isBlank(r.getCell(1).getStringCellValue()))
                        && (r.getCell(2) == null || StringUtils.isBlank(r.getCell(2).getStringCellValue()))) {
                    continue;//若一条记录前三个数据同时为空，则判断为无效记录
                }
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''姓名''为空！");
            }
            info.setName(r.getCell(0).getStringCellValue());
            if (info.getName().length() > 30) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''姓名''长度超过30！");
            }
            if (UserTypeEnum.POLICE.getValue().equals(userType)) { //民警
                if (r.getCell(1) == null || StringUtils.isBlank(r.getCell(1).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警号''为空！");
                }
                if (r.getCell(1).getStringCellValue().length() != 6
                        || !StringUtils.isNumeric(r.getCell(1).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警号''格式为6位数字，如''000001''！");
                }
            } else if (UserTypeEnum.EXTERNAL.getValue().equals(userType)) { //外部人员
                if (r.getCell(1) == null || StringUtils.isBlank(r.getCell(1).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''编号''为空！");
                }
                if (r.getCell(1).getStringCellValue().length() != 8
                        || !"WB".equals(r.getCell(1).getStringCellValue().substring(0, 2))
                        || !StringUtils.isNumeric(r.getCell(1).getStringCellValue().substring(2, 6))) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''编号''格式为''WB''加6位数字，如''WB000001''！");
                }
            } else if (UserTypeEnum.HELPER.getValue().equals(userType)) { //辅警
/*                if (r.getCell(1) == null || StringUtils.isBlank(r.getCell(1).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'编号\'\'为空！");
                }
                if (r.getCell(1).getStringCellValue().length() != 8
                        || !"FJ".equals(r.getCell(1).getStringCellValue().substring(0, 2))
                        || !StringUtils.isNumeric(r.getCell(1).getStringCellValue().substring(2, 6))) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'编号\'\'格式为\'\'FJ\'\'加6位数字，如\'\'FJ000001\'\'！");
                }*/
            }
            if(r.getCell(1)!=null){
                r.getCell(1).setCellType(Cell.CELL_TYPE_STRING);
                info.setPoliceNumber(r.getCell(1).getStringCellValue());
            }else{
                info.setPoliceNumber("");
            }
            if (r.getCell(2) == null || StringUtils.isBlank(r.getCell(2).getStringCellValue())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''身份证''为空！");
            }
            if (!UserUtils.isIDNumber(r.getCell(2).getStringCellValue())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''身份证''格式错误！");
            }
            r.getCell(2).setCellType(Cell.CELL_TYPE_STRING);
            info.setIdcard(r.getCell(2).getStringCellValue());
            userBybrith(info);
            int cellNum2 = 3;
            if (UserTypeEnum.POLICE.getValue().equals(userType) || UserTypeEnum.HELPER.getValue().equals(userType)) {
                if (r.getCell(cellNum2) == null || StringUtils.isBlank(r.getCell(cellNum2).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''单位机构代码''为空！");
                }
                info.setOrgCode(r.getCell(cellNum2++).getStringCellValue());
                if (info.getOrgCode().length() > 32) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''单位机构代码''长度超过32！");
                }
            }
            //工作号码
            if (r.getCell(cellNum2) == null || StringUtils.isBlank(r.getCell(cellNum2).getStringCellValue())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''手机号码（接收短信验证码）''为空！");
            }
            r.getCell(cellNum2).setCellType(Cell.CELL_TYPE_STRING);
            info.setMobileWork(r.getCell(cellNum2++).getStringCellValue());
            if (info.getMobileWork().length() != 11 || !StringUtils.isNumeric(info.getMobileWork())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''手机号码（接收短信验证码）''格式为11位数字！");
            }
            //
            if (r.getCell(cellNum2) == null || StringUtils.isBlank(r.getCell(cellNum2).getStringCellValue())) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''民族''为空！");
            }
            r.getCell(cellNum2).setCellType(Cell.CELL_TYPE_STRING);
            info.setNation(r.getCell(cellNum2++).getStringCellValue());
            if (info.getNation().length() > 2) {
                throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''民族''长度超过2！");
            }
            //
           /* if (r.getCell(cellNum2) != null && StringUtils.isNotBlank(r.getCell(cellNum2).getStringCellValue())) {
                r.getCell(cellNum2).setCellType(Cell.CELL_TYPE_STRING);
                info.setQqAccount(r.getCell(cellNum2++).getStringCellValue());
                if (info.getQqAccount().length() > 30) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'QQ号\'\'长度超过30！");
                }
            }
            //
            if (r.getCell(cellNum2) != null && StringUtils.isNotBlank(r.getCell(cellNum2).getStringCellValue())) {
                info.setEmail(r.getCell(cellNum2++).getStringCellValue());
                if (!UserUtils.checkEmaile(info.getEmail())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'e-mail\'\'格式不正确！");
                }
                if (info.getEmail().length() > 60) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'e-mail\'\'长度超过30！");
                }
            }
            //
            if (r.getCell(cellNum2) != null && StringUtils.isNotBlank(r.getCell(cellNum2).getStringCellValue())) {
                r.getCell(cellNum2).setCellType(Cell.CELL_TYPE_STRING);
                info.setWxAccount(r.getCell(cellNum2++).getStringCellValue());
                if (info.getWxAccount().length() > 30) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'微信\'\'长度超过30！");
                }
            }
            //
            if (r.getCell(cellNum2) != null && StringUtils.isNotBlank(r.getCell(cellNum2).getStringCellValue())) {
                r.getCell(cellNum2).setCellType(Cell.CELL_TYPE_STRING);
                info.setPhone(r.getCell(cellNum2++).getStringCellValue());
                if (info.getPhone().length() > 30) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'座机\'\'长度超过30！");
                }
                int startPosition = info.getPhone().indexOf("-");
                int endPosition = info.getPhone().lastIndexOf("-");
                if (startPosition == endPosition) {
                    String[] numbers = info.getPhone().split("-");
                    for (String number : numbers) {
                        boolean isNumber3 = StringUtils.isNumeric(number);
                        if (!isNumber3)
                            throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'座机\'\'为无效的数值！");
                    }
                } else {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'座机\'\'为无效的数值！");
                }
            }
            //
            if (r.getCell(cellNum2) != null && StringUtils.isNotBlank(r.getCell(cellNum2).getStringCellValue())) {
                info.setAddress(r.getCell(cellNum2++).getStringCellValue());
                if (info.getAddress().length() > 60) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'地址\'\'长度超过60！");
                }
            }*/
            //
            info.setUserType(userType);
            info.setDeleted("0");
            if (UserTypeEnum.HELPER.getValue().equals(userType)) {//辅警
                info.setUserType(UserTypeEnum.HELPER.getValue());
                if (r.getCell(6) == null || StringUtils.isBlank(r.getCell(6).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''人员类型''为空！");
                }
                r.getCell(6).setCellType(Cell.CELL_TYPE_STRING);
                info.setAuxiliaryType(r.getCell(6).getStringCellValue());
                //
                if (r.getCell(7) == null || StringUtils.isBlank(r.getCell(7).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警种''为空！");
                }
                r.getCell(7).setCellType(Cell.CELL_TYPE_STRING);
                info.setPoliceCategory(r.getCell(7).getStringCellValue());
                if (info.getPoliceCategory().length() > 10) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警种''长度超过10！");
                }
                if (r.getCell(8) == null || StringUtils.isBlank(r.getCell(8).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''人员状态(表码)''为空！");
                }
                r.getCell(8).setCellType(Cell.CELL_TYPE_STRING);
                info.setUserStatus(r.getCell(8).getStringCellValue());
                //
//                if (info.getPoliceCategory().equals("1") && (r.getCell(13) == null || StringUtils.isBlank(r.getCell(13).getStringCellValue()))) {
//                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'业务域\'\'为空！");
//                }
/*                if (info.getPoliceCategory().equals("2006")) {
                    r.getCell(13).setCellType(Cell.CELL_TYPE_STRING);
                    info.setPoliceBusiness(r.getCell(13).getStringCellValue());
                    if (info.getPoliceBusiness().length() > 200) {
                        throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'业务域\'\'长度超过200！");
                    }
                }*/
            } else if (UserTypeEnum.POLICE.getValue().equals(userType)) {//警员
                info.setUserType(UserTypeEnum.POLICE.getValue());
                if (r.getCell(6) != null && StringUtils.isNotBlank(r.getCell(6).getStringCellValue())) {
                    r.getCell(6).setCellType(Cell.CELL_TYPE_STRING);
                    info.setMobilePrivate(r.getCell(6).getStringCellValue());
                    if (info.getMobilePrivate().length() != 11 || !StringUtils.isNumeric(info.getMobilePrivate())) {
                        throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''手机号码（私人）''格式为11位数字！");
                    }
                }
                //
                if (r.getCell(7) == null || StringUtils.isBlank(r.getCell(7).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''人员类型''为空！");
                }
                r.getCell(7).setCellType(Cell.CELL_TYPE_STRING);
                info.setManType(r.getCell(7).getStringCellValue());
                if (info.getManType().length() > 5) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''人员类型''长度超过5！");
                }
                //
                if (r.getCell(8) == null || StringUtils.isBlank(r.getCell(8).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警衔''为空！");
                }
                r.getCell(8).setCellType(Cell.CELL_TYPE_STRING);
                info.setRank(r.getCell(8).getStringCellValue());
                if (info.getRank().length() > 10) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警衔''长度超过10！");
                }
                //
                if (r.getCell(9) == null || StringUtils.isBlank(r.getCell(9).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''职级''为空！");
                }
                r.getCell(9).setCellType(Cell.CELL_TYPE_STRING);
                info.setTitle(r.getCell(9).getStringCellValue());
                if (info.getTitle().length() > 20) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''职级''长度超过20！");
                }
                //职务
                if (r.getCell(10) == null || StringUtils.isBlank(r.getCell(10).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''职务''为空！");
                }
                r.getCell(10).setCellType(Cell.CELL_TYPE_STRING);
                info.setPostType(r.getCell(10).getStringCellValue());
                if (info.getPostType().length() > 50) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''职务''长度超过50！");
                }
                //在职类型
                if (r.getCell(11) == null || StringUtils.isBlank(r.getCell(11).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''在职类型''为空！");
                }
                r.getCell(11).setCellType(Cell.CELL_TYPE_STRING);
                info.setJobType(r.getCell(11).getStringCellValue());
                if (info.getJobType().length() > 4) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''在职类型''长度超过4！");
                }
                //
                if (r.getCell(12) == null || StringUtils.isBlank(r.getCell(12).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警种''为空！");
                }
                r.getCell(12).setCellType(Cell.CELL_TYPE_STRING);
                info.setPoliceCategory(r.getCell(12).getStringCellValue());
                if (info.getPoliceCategory().length() > 10) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''警种''长度超过10！");
                }
//                if (info.getPoliceCategory().equals("1") && (r.getCell(18) == null || StringUtils.isBlank(r.getCell(18).getStringCellValue()))) {
//                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'业务域\'\'为空！");
//                }
               /* if (info.getPoliceCategory().equals("2006")) {
                    r.getCell(18).setCellType(Cell.CELL_TYPE_STRING);
                    info.setPoliceBusiness(r.getCell(18).getStringCellValue());
                    if (info.getPoliceBusiness().length() > 200) {
                        throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行\'\'业务域\'\'长度超过200！");
                    }
                }*/
            } else if (UserTypeEnum.EXTERNAL.getValue().equals(userType)) {//施工人员
                info.setUserType(UserTypeEnum.EXTERNAL.getValue());
                info.setUserStatus(UserStatusEnum.STARTUSING.getValue());
                if (r.getCell(10) == null || StringUtils.isBlank(r.getCell(10).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''厂商代码''为空！");
                }
                info.setCompany(r.getCell(10).getStringCellValue());
                if (info.getCompany().length() > 100) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''厂商代码''长度超过100！");
                }
                //
                if (r.getCell(11) == null || StringUtils.isBlank(r.getCell(11).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''公司负责人''为空！");
                }
                info.setCompanyPerson(r.getCell(11).getStringCellValue());
                if (info.getCompanyPerson().length() > 20) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''公司负责人''长度超过20！");
                }
                //
                if (r.getCell(12) == null || StringUtils.isBlank(r.getCell(12).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''公司负责人电话''为空！");
                }
                r.getCell(12).setCellType(Cell.CELL_TYPE_STRING);
                info.setPersonMobile(r.getCell(12).getStringCellValue());
                if (info.getPersonMobile().length() > 11) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''公司负责人电话''长度超过11！");
                }
                //
                if (r.getCell(13) == null || StringUtils.isBlank(r.getCell(13).getStringCellValue())) {
                    throw new NumberFormatException("第" + (r.getRowNum() + 1) + "行''施工项目代码''为空！");
                }
                info.setProject(r.getCell(13).getStringCellValue());
            }
            //警员归属默认广东省
            info.setArea("440000");
            temp.add(info);
        }
        inputStream.close();
        return temp;
    }

    public static List<GovUserInfo> loadScoreGovInfo(InputStream inputStream) throws IOException {
        List<GovUserInfo> temp = new ArrayList();
        //根据指定的文件输入流导入Excel从而产生Workbook对象
        Workbook wb0 = new HSSFWorkbook(inputStream);
        //获取Excel文档中的第一个表单
        Sheet sht0 = wb0.getSheetAt(0);
        for (Row r : sht0) {
            if (r.getRowNum() == 0) {
                checkRowTitle(r);
                continue;
            }
            if (checkRowDataValid(r)) {
                continue;
            }
            GovUserInfo govUserInfo = getGovUserInfo(r);
            govUserByBirth(govUserInfo);
            temp.add(govUserInfo);
        }
        return temp;
    }

    /**
     * 检查行标题
     *
     * @param r
     */
    private static void checkRowTitle(Row r) {
        if (!r.getCell(0).getStringCellValue().equals("姓名")) {
            throw new NumberFormatException("缺少''姓名''列！");
        }
        if (!r.getCell(1).getStringCellValue().equals("编号")) {
            throw new NumberFormatException("缺少''编号''列！");
        }
        if (!r.getCell(2).getStringCellValue().equals("身份证")) {
            throw new NumberFormatException("缺少''身份证''列！");
        }
        if (!r.getCell(3).getStringCellValue().equals("单位机构代码")) {
            throw new NumberFormatException("缺少''单位机构代码''列！");
        }
        if (!r.getCell(4).getStringCellValue().equals("手机号码")) {
            throw new NumberFormatException("缺少''手机号码''列！");
        }
        if (!r.getCell(5).getStringCellValue().equals("民族（表码）")) {
            throw new NumberFormatException("缺少''民族（表码）''列！");
        }
        if (!r.getCell(6).getStringCellValue().equals("QQ号")) {
            throw new NumberFormatException("缺少''QQ号''列！");
        }
        if (!r.getCell(7).getStringCellValue().equals("e-mail")) {
            throw new NumberFormatException("缺少''e-mail''列！");
        }
        if (!r.getCell(8).getStringCellValue().equals("微信号")) {
            throw new NumberFormatException("缺少''微信号''列！");
        }
        if (!r.getCell(9).getStringCellValue().equals("座机")) {
            throw new NumberFormatException("缺少''座机''列！");
        }
        if (!r.getCell(10).getStringCellValue().equals("联系地址")) {
            throw new NumberFormatException("缺少''联系地址''列！");
        }
        if (!r.getCell(11).getStringCellValue().equals("职务")) {
            throw new NumberFormatException("缺少''职务''列！");
        }
    }

    /**
     * 返回行数据转换后的政府人员
     *
     * @param r
     * @return
     */
    private static GovUserInfo getGovUserInfo(Row r) {
        GovUserInfo govUserInfo = new GovUserInfo();
        for (int i = 0; i < 14; i++) {
            Cell cell = r.getCell(i);
            checkCellHasData(cell, i, r.getRowNum());
            checkCellDataValid(cell, i, r.getRowNum());
            setGovUserInfo(govUserInfo, cell, i);
        }
        govUserInfo.setCreateTime(new Date());
        govUserInfo.setDeleted(YesNotEnum.NO.getValue());
        return govUserInfo;
    }

    /**
     * 检查必填单元是否有值
     *
     * @param cell
     * @param num
     */
    private static void checkCellHasData(Cell cell, int num, int row) {
        if (!(cell == null || StringUtils.isBlank(cell.getStringCellValue()))) {
            return;
        }
        switch (num) {
            case 0:
                throw new NumberFormatException("第" + (row + 1) + "行''姓名''为空！");
            case 1:
                throw new NumberFormatException("第" + (row + 1) + "行''编号''为空！");
            case 2:
                throw new NumberFormatException("第" + (row + 1) + "行''身份证''为空！");
            case 3:
                throw new NumberFormatException("第" + (row + 1) + "行''单位机构代码''为空！");
            case 4:
                throw new NumberFormatException("第" + (row + 1) + "行''手机号码''为空！");
            case 5:
                throw new NumberFormatException("第" + (row + 1) + "行''民族（表码）''为空！");
            case 11:
                throw new NumberFormatException("第" + (row + 1) + "行''职务''为空！");
            default:
                break;
        }
    }

    /**
     * 检查单元值是否合法
     *
     * @param cell
     * @param num
     */
    private static void checkCellDataValid(Cell cell, int num, int row) {
        if (cell == null)
            return;
        String value = cell.getStringCellValue();
        switch (num) {
            case 0:
                if (value.length() > 15) {
                    throw new NumberFormatException("第" + (row + 1) + "行''姓名''长度大于15！");
                }
                break;
            case 1:
                if (!StringUtils.equals(value.substring(0, 2), "ZF")) {
                    throw new NumberFormatException("第" + (row + 1) + "行''编号''不是'ZF'开头！");
                }
                if (value.substring(2).length() != 6) {
                    throw new NumberFormatException("第" + (row + 1) + "行''编号''长度应该为8位！");
                }
                break;
            case 2:
                if (!UserUtils.isIDNumber(value)) {
                    throw new NumberFormatException("第" + (row + 1) + "行''身份证''号码格式错误！");
                }
                break;
            case 4:
                if (value.length() != 11 && StringUtils.isNumeric(value)) {
                    throw new NumberFormatException("第" + (row + 1) + "行''手机号码''应该为11位数字！");
                }
                break;
            case 5:
                if (value.length() > 2) {
                    throw new NumberFormatException("第" + (row + 1) + "行''民族（表码）''应该为2位数字！");
                }
                break;
            case 6:
                if (value.length() > 30) {
                    throw new NumberFormatException("第" + (row + 1) + "行''QQ''长度超过30！");
                }
                break;
            case 7:
                if (!UserUtils.checkEmaile(value)) {
                    throw new NumberFormatException("第" + (row + 1) + "行''e-mail''格式不正确！");
                }
                if (value.length() > 60) {
                    throw new NumberFormatException("第" + (row + 1) + "行''e-mail''长度超过30！");
                }
                break;
            case 8:
                if (value.length() > 30) {
                    throw new NumberFormatException("第" + (row + 1) + "行''微信''长度超过30！");
                }
                break;
            case 9:
                if (value.length() > 30) {
                    throw new NumberFormatException("第" + (row + 1) + "行''座机''长度超过30！");
                }
                int startPosition = value.indexOf("-");
                int endPosition = value.lastIndexOf("-");
                if (startPosition == endPosition) {
                    String[] numbers = value.split("-");
                    for (String number : numbers) {
                        boolean isNumber3 = StringUtils.isNumeric(number);
                        if (!isNumber3)
                            throw new NumberFormatException("第" + (row + 1) + "行''座机''为无效的数值！");
                    }
                } else {
                    throw new NumberFormatException("第" + (row + 1) + "行''座机''为无效的数值！");
                }
                break;
            case 10:
                if (value.length() > 60) {
                    throw new NumberFormatException("第" + (row + 1) + "行''地址''长度超过60！");
                }
                break;
            case 11:
                if (value.length() > 30) {
                    throw new NumberFormatException("第" + (row + 1) + "行''职务''长度超过30！");
                }
                break;
            default:
                break;

        }
    }

    /**
     * 将单元值赋给政府人员类
     *
     * @param govUserInfo
     * @param cell
     * @param num
     */
    private static void setGovUserInfo(GovUserInfo govUserInfo, Cell cell, int num) {
        if (cell == null) {
            return;
        }
        switch (num) {
            case 0:
                govUserInfo.setName(cell.getStringCellValue());
                break;
            case 1:
                govUserInfo.setUserNumber(cell.getStringCellValue());
                break;
            case 2:
                govUserInfo.setIdcard(cell.getStringCellValue());
                break;
            case 3:
                govUserInfo.setGovId(cell.getStringCellValue());
                break;
            case 4:
                govUserInfo.setMobileWork(cell.getStringCellValue());
                break;
            case 5:
                govUserInfo.setNation(cell.getStringCellValue());
                break;
            case 6:
                govUserInfo.setQqAccount(cell.getStringCellValue());
                break;
            case 7:
                govUserInfo.setEmail(cell.getStringCellValue());
                break;
            case 8:
                govUserInfo.setWxAccount(cell.getStringCellValue());
                break;
            case 9:
                govUserInfo.setPhone(cell.getStringCellValue());
                break;
            case 10:
                govUserInfo.setAddress(cell.getStringCellValue());
                break;
            case 11:
                govUserInfo.setPost(cell.getStringCellValue());
                break;
            default:
                break;
        }
    }

    /**
     * 检查行数据有效性，前三个单元为空值则此行数据无效
     *
     * @param r
     * @return
     */
    private static boolean checkRowDataValid(Row r) {
        boolean flag;
        flag = (r.getCell(0) == null || StringUtils.isBlank(r.getCell(0).getStringCellValue())) &&
                (r.getCell(1) == null || StringUtils.isBlank(r.getCell(1).getStringCellValue())) &&
                (r.getCell(2) == null || StringUtils.isBlank(r.getCell(2).getStringCellValue()));
        return flag;
    }

    /**
     * 生日，性别
     *
     * @param userInfo
     * @return
     */

    public static UserInfo userBybrith(UserInfo userInfo) {
        if (StringUtils.isNotEmpty(userInfo.getIdcard())) {
            String year = userInfo.getIdcard().substring(6, 10);
            String mon = userInfo.getIdcard().substring(10, 12);
            String d = userInfo.getIdcard().substring(12, 14);
            String date = year + mon + d;
            TimeZone.setDefault(TimeZone.getTimeZone("GMT+08"));
            userInfo.setBirth(date);
            int sex = Integer.parseInt(userInfo.getIdcard().substring(16, 17));
            userInfo.setSex((sex % 2 == 0) ? "2" : "1");
        }
        return userInfo;
    }

    public static GovUserInfo govUserByBirth(GovUserInfo govUserInfo) {
        try {
            if (StringUtils.isNotEmpty(govUserInfo.getIdcard())) {
                String year = govUserInfo.getIdcard().substring(6, 10);
                String mon = govUserInfo.getIdcard().substring(10, 12);
                String d = govUserInfo.getIdcard().substring(12, 14);
                String date = year + "-" + mon + "-" + d;
                TimeZone.setDefault(TimeZone.getTimeZone("GMT+08"));
                DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
                govUserInfo.setBirth(df.parse(date));
                int sex = Integer.valueOf(govUserInfo.getIdcard().substring(16, 17));
                govUserInfo.setSex((sex % 2 == 0) ? "2" : "1");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return govUserInfo;
    }
}
