package com.dragoninfo.dcuc.authweb.util;

import com.dragoninfo.dcuc.authweb.restcontroller.user.vo.PostInfoVo;
import com.dragoninfo.dcuc.org.entity.OrgInfo;
import com.dragoninfo.dcuc.org.facade.IOrgInfoFacade;
import com.dragoninfo.dcuc.user.user.entity.PostInfo;
import com.dragoninfo.dcuc.user.user.entity.UserInfo;
import com.dragoninfo.dcuc.user.user.entity.UserInfoView;
import com.dragoninfo.dcuc.user.user.enumresources.AuxiliaryTypeEnum;
import com.dragoninfo.dcuc.user.user.enumresources.JobTypeEnum;
import com.dragoninfo.dcuc.user.user.enumresources.UserTypeEnum;
import com.dragoninfo.dcuc.user.user.facade.IUserInfoFacade;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.bean.BeanUtils;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import net.sf.ezmorph.bean.MorphDynaBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by Administrator on 2018/7/4.
 */
public class UserUtils {
    private static Logger logger = LoggerFactory.getLogger(UserUtils.class);

    public static String getSysUserListId() {
        return "00000000000000000000000000000000";
    }


    /**
     * 职务信息
     *
     * @param json
     * @return
     */
    public static List<PostInfo> fromJson(String json) {
        List<PostInfo> postInfos = new ArrayList<PostInfo>();
        Object[] objects = JsonUtils.toArray(json);
        for (int i = 0; i < objects.length; i++) {
            MorphDynaBean morphDynaBean = (MorphDynaBean) objects[i];
            PostInfo pInfo = new PostInfo();
            String id = "";
            if (morphDynaBean.toString().indexOf("id") != -1) {
                id = (String) morphDynaBean.get("id");
                pInfo.setId(id);
            }
            String orgId = (String) morphDynaBean.get("orgId");
            String postType = (String) morphDynaBean.get("postType");
            String jobType = (String) morphDynaBean.get("jobType");
            pInfo.setOrgId(orgId);
            pInfo.setPostType(postType);
            pInfo.setJobType(jobType);
            postInfos.add(pInfo);
        }
        return postInfos;
    }

    /**
     * 正则身份证校验
     *
     * @param IDNumber
     * @return
     */
    public static boolean isIDNumber(String IDNumber) {
        if (IDNumber == null || "".equals(IDNumber)) {
            return false;
        }
        // 定义判别用户身份证号的正则表达式（15位或者18位，最后一位可以为字母）
        String regularExpression = "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" +
                "(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)";
        //假设18位身份证号码:41000119910101123X  410001 19910101 123X
        //^开头
        //[1-9] 第一位1-9中的一个      4
        //\\d{5} 五位数字           10001（前六位省市县地区）
        //(18|19|20)                19（现阶段可能取值范围18xx-20xx年）
        //\\d{2}                    91（年份）
        //((0[1-9])|(10|11|12))     01（月份）
        //(([0-2][1-9])|10|20|30|31)01（日期）
        //\\d{3} 三位数字            123（第十七位奇数代表男，偶数代表女）
        //[0-9Xx] 0123456789Xx其中的一个 X（第十八位为校验值）
        //$结尾

        //假设15位身份证号码:410001910101123  410001 910101 123
        //^开头
        //[1-9] 第一位1-9中的一个      4
        //\\d{5} 五位数字           10001（前六位省市县地区）
        //\\d{2}                    91（年份）
        //((0[1-9])|(10|11|12))     01（月份）
        //(([0-2][1-9])|10|20|30|31)01（日期）
        //\\d{3} 三位数字            123（第十五位奇数代表男，偶数代表女），15位身份证不含X
        //$结尾
        boolean matches = IDNumber.matches(regularExpression.trim());
        //判断第18位校验值
        if (matches) {
            if (IDNumber.length() == 18) {
                try {
                    char[] charArray = IDNumber.toCharArray();
                    //前十七位加权因子
                    int[] idCardWi = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
                    //这是除以11后，可能产生的11位余数对应的验证码
                    String[] idCardY = {"1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"};
                    int sum = 0;
                    for (int i = 0; i < idCardWi.length; i++) {
                        int current = Integer.parseInt(String.valueOf(charArray[i]));
                        int count = current * idCardWi[i];
                        sum += count;
                    }
                    char idCardLast = charArray[17];
                    int idCardMod = sum % 11;
                    if (idCardY[idCardMod].toUpperCase().equals(String.valueOf(idCardLast).toUpperCase())) {
                        return true;
                    } else {
                        System.out.println("身份证最后一位:" + String.valueOf(idCardLast).toUpperCase() +
                                "错误,正确的应该是:" + idCardY[idCardMod].toUpperCase());
                        return false;
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println("异常:" + IDNumber);
                    return false;
                }
            }

        }
        return matches;
    }

    public static boolean checkEmaile(String emaile) {
        String RULE_EMAIL = "^\\w+((-\\w+)|(\\.\\w+))*\\@[A-Za-z0-9]+((\\.|-)[A-Za-z0-9]+)*\\.[A-Za-z0-9]+$";
        //正则表达式的模式 编译正则表达式
        Pattern p = Pattern.compile(RULE_EMAIL);
        //正则表达式的匹配器
        Matcher m = p.matcher(emaile);
        //进行正则匹配\
        return m.matches();
    }

    /**
     * 放入用户部门、层级、所在机构区划
     *
     * @param list
     * @param info
     * @return
     */
    public static UserInfo setUserOrg(IOrgInfoFacade orgInfoFacade, List<PostInfo> list, UserInfo info) {
        String postId = "";
        String orgId = "";
        int jobType = Integer.MAX_VALUE;
        for (PostInfo postinfo : list) {
            String type = postinfo.getJobType();
            //确定警员的单位依次按 JobTypeEnum code 从小到大
            if (StringUtils.isNotEmpty(type) && Integer.parseInt(type) < jobType) {
                orgId = postinfo.getOrgId();
                jobType = Integer.parseInt(postinfo.getJobType());
            }
            if (StringUtils.isNotEmpty(postinfo.getId())) {
                postId += postinfo.getId() + ",";
            }
            //设置职位机构名称
            OrgInfo orgInfo = orgInfoFacade.detail(postinfo.getOrgId());
            postinfo.setOrgName(orgInfo.getFullName());
            if (JobTypeEnum.FP.getValue().equals(type)) {
                info.setUserType(UserTypeEnum.HELPER.getValue());
                info.setAuxiliaryType(AuxiliaryTypeEnum.FP.getValue());
                orgId = postinfo.getOrgId();
                break;
            }
        }
        OrgInfo orgInfo = orgInfoFacade.detail(orgId);
        info.setOrgId(orgId);
        info.setOrgName(orgInfo.getFullName());
        //添加层级
        info.setOrgLevel(orgInfo.getOrgLevel());
        info.setOrgCode(orgInfo.getCode());
        info.setJobType(postId);
        info.setDeleted("0");
        return info;
    }

    /**
     * 对象复制
     *
     * @param obj1 需复制的对象
     * @param obj2 被复制的对象
     * @return
     */
    public static <T> T copyObject(T obj1, T obj2) {
        try {
            BeanUtils.copyProperties(obj1,obj2);
        } catch (IllegalAccessException e) {
            logger.error("属性拷贝异常",e);
        } catch (InvocationTargetException e) {
            logger.error("属性拷贝异常",e);
        }
        return obj1;
    }




    public static List<PostInfo> copyObjectList(List<PostInfoVo> postInfos) {

        List<PostInfo> objects = new ArrayList<>();
        if (CollectionUtils.isEmpty(postInfos))return objects;
        for (PostInfoVo postInfoVo : postInfos) {
            PostInfo postInfo1 = new PostInfo();
            try {
                BeanUtils.copyProperties(postInfo1,postInfoVo);
            } catch (IllegalAccessException e) {
                logger.error("属性拷贝异常",e);
            } catch (InvocationTargetException e) {
                logger.error("属性拷贝异常",e);
            }
            objects.add(postInfo1);
        }
        return objects;
    }

    public static List<PostInfoVo> copyObjectByVo(List<PostInfo> postInfos) {
        List<PostInfoVo> objects = new ArrayList<>();
        for (PostInfo postInfo : postInfos) {
            PostInfoVo postInfo1 = new PostInfoVo();
            try {
                BeanUtils.copyProperties(postInfo1,postInfo);
            } catch (IllegalAccessException e) {
                logger.error("属性拷贝异常",e);
            } catch (InvocationTargetException e) {
                logger.error("属性拷贝异常",e);
            }
            objects.add(postInfo1);
        }
        return objects;
    }

    /**
     * 日期转换
     * @param date
     * @return
     */
    public static String transformDate(Date date){
        if (date==null)return "";
        SimpleDateFormat dateformat=new SimpleDateFormat("yyyy-MM-dd");
        String newdate=dateformat.format(date);
        return newdate;
    }
    /**
     * 8,14位日期转换
     * @param date
     * @return
     */
    public static String birthTransform(String date,String bit){
        String newdate = date;
        if (StringUtils.isEmpty(date))return "";
        if(bit.equals("8"))
        {/*8位日期转换*/
            newdate=date.substring(0,4)+"-"+ date.substring(4,6)+"-"+date.substring(6,8);
        } if(bit.equals("14")){
            /*14位日期转换*/
            newdate=date.substring(0,4)+"-"+ date.substring(4,6)+"-"+date.substring(6,8)+" "+date.substring(8,10)+":"+date.substring(10,12)+":"+date.substring(12,14);
        }if(bit.equals("10")){
            /*14位日期转换*/
            newdate=date.substring(0,4)+"-"+ date.substring(4,6)+"-"+date.substring(6,8)+" "+date.substring(8,10)+":"+date.substring(10,12)+":"+date.substring(12,14);
        }
        return newdate;
    }



    /**
     * 验证身份证

     * @param info
     * @return
     */
    public static Boolean verificaIdcard(UserInfo info, IUserInfoFacade userInfoFacade) {
        if (StringUtils.isNotEmpty(info.getIdcard())){
            Map map=new HashMap();
            map.put("userId",info.getId());
            map.put("idcard",info.getIdcard());
            ResponseStatus responseStatus = userInfoFacade.checkIdcard(map);
            return !"200".equals(responseStatus.getStatusCode());
        }
        return false;
    }

    /**
     * 验证姓名

     * @param info
     * @return
     */
    public static Boolean verificaName(UserInfo info){
        return "ADMIN".equals(info.getName().trim());
    }




}
